
# 
#   ______                           __        ____    _      __  __     
#  / ___(_)__ ___ _  ___  ___ ____  / /__     / __/___(_)_ __/ /_/ /____ 
# / (_ / / _ `/  ' \/ _ \/ _ `/ _ \/ / _ \   _\ \/ __/ / // / __/ __/ _ \
# \___/_/\_,_/_/_/_/ .__/\_,_/\___/_/\___/  /___/\__/_/\_,_/\__/\__/\___/
#                 /_/                                                   
# http://www.giampaolo-sciutto.it/

# Name Script : WorkstationsWithBitLocker.ps1
# Descrizione : Script che legge Active Directory e genera un .CSV con la data della workstation Bitlockata (terza colonna)
# 	*** Importante - utilizzare da un account che abbia i diritti di lettura in ADSI Edit del valore "msFVE-RecoveryInformation" ***
# Versione    : 1.1
# Requisiti   : Import-Module ActiveDirectory - specificare la OU
# Piattaforme Verificate : Windows 2008R2 - Windows 7 - Windows 2012
#
# Modify by Giampaolo Sciutto
# Date Modify: 27/07/2016
#
#

# Definire la Organization Unit desiderata sotto la voce "$TargetOU" qui sotto:
#>

$TargetOU = 'OU=HeadQuarter,DC=costa,DC=it'

Import-Module ActiveDirectory


    [string]$SearchBase = "$TargetOU"

Try { Import-Module ActiveDirectory -ErrorAction Stop }
Catch { Write-Warning "Unable to load Active Directory module because $($Error[0])"; Exit }


Write-Verbose "Getting Workstations..." -Verbose
$Computers = Get-ADComputer -Filter * -SearchBase $SearchBase -Properties LastLogonDate
$Count = 1
$Results = ForEach ($Computer in $Computers)
{
    Write-Progress -Id 0 -Activity "Searching Computers for BitLocker" -Status "$Count of $($Computers.Count)" -PercentComplete (($Count / $Computers.Count) * 100)
    New-Object PSObject -Property @{
        ComputerName = $Computer.Name
        LastLogonDate = $Computer.LastLogonDate 
        BitLockerPasswordSet = Get-ADObject -Filter "objectClass -eq 'msFVE-RecoveryInformation'" -SearchBase $Computer.distinguishedName -Properties msFVE-RecoveryPassword,whenCreated | Sort whenCreated -Descending | Select -First 1 | Select -ExpandProperty whenCreated
    }
    $Count ++
}
Write-Progress -Id 0 -Activity " " -Status " " -Completed

$ReportPath = Join-Path (Split-Path $MyInvocation.MyCommand.Path) -ChildPath "WorkstationsWithBitLocker.csv"
Write-Verbose "Building the report..." -Verbose
$Results | Select ComputerName,LastLogonDate,BitLockerPasswordSet | Sort ComputerName | Export-Csv $ReportPath -NoTypeInformation
Write-Verbose "Report saved at: $ReportPath" -Verbose